### MANAGED BY PUPPET ###

<% @options.sort.map do |k,v| -%>
<%   if v.is_a?(Hash) -%>
[<%=   k %>]
<%     v.sort.map do |ki, vi| -%>
<%       if ki == 'ssl-disable' or (ki =~ /^ssl/ and v['ssl-disable'] == true) or (ki =~ /^ssl-/ and v['ssl'] == false) -%>
<%         next %>
<%       elsif vi == true or vi == '' -%>
<%=        ki %>
<%       elsif vi.is_a?(Array) -%>
<%         vi.each do |vii| -%>
<%=          ki %> = <%= vii %>
<%         end -%>
<%       elsif ![nil, '', :undef].include?(vi) -%>
<%=        ki %> = <%= vi %>
<%       end -%>
<%     end -%>
<%   end %>
<% end -%>

<% if @includedir and @includedir != '' %>
!includedir <%= @includedir %>
<% end %>
