### MANAGED BY PUPPET ###

<% %w(mysql client mysqldump mysqladmin mysqlcheck).each do |section| %>
[<%= section -%>]
user=root
host=localhost
<% if @root_password_set -%>
password='<%= @root_password %>'
<% end -%>
socket=<%= @options['client']['socket'] %>
<% end %>
